class Calculator:
    '''
    A simple calculator class that can perform basic arithmetic operations like:
    - add for addition
    - sub for subtraction
    - root for getting the nth root of a number
    - reset for resetting the result to 0
    '''
    def __init__(self, number_input=0):
        self._number_input = number_input
        self.result = 0  #calculator's memory, stores the result of the last operation

    @property
    def number_input(self):
        '''
        Getter for the number_input attribute
        '''
        return self._number_input

    @number_input.setter
    def number_input(self, value):
        '''
        Setter for the number_input attribute
        value: float or integer
        '''
        if not isinstance(value, (int, float)):
            raise TypeError("Input must be a number (float or integer)")
        self._number_input = value

    def add(self, number):
        '''
        Adds the input number to the result
        '''
        self.result += number
        return self.result

    def sub(self, number):
        '''
        Subtracts the input number from the result
        '''
        self.result -= number
        return self.result

    def root(self, number):
        '''
        Returns the nth root of the result

        Cannot have root exponent of 0
        Cannot return real numbers for even roots of negative numbers

        '''
        if number == 0:
            raise ValueError("The root exponent cannot be 0")

        root_exponenet = abs(number) #absolute value for valid root exponent

        if self.result < 0 and root_exponenet % 2 == 0:
            raise ValueError("Cannot return real numbers for even roots of negative numbers")

        if self.result < 0 and root_exponenet % 2 != 0:
            root_value = - (abs(self.result) ** (1 / root_exponenet))
        else:
            root_value = self.result ** (1 / root_exponenet)

        if number < 0:
            root_value = -root_value #changing sign after calculations

        self.result = root_value

        # For readability, if result is float or integer
        if root_value.is_integer():
            return f"{root_value:.0f}"
        return f"{root_value:.3f}"

    def reset(self):
        ''' 
        Resets the result(memory) to 0. 
        '''
        self.result = 0

    def __str__(self):
        if self.result.is_integer():
            return f"{self.result:.0f}"
        return f"{self.result:.3f}"
